@@ -16,20 +16,15 @@ class ActionCable::Connection::MultipleIdentifiersTest < ActionCable::TestCase
 
   test "multiple connection identifiers" do
     run_in_eventmachine do
-      open_connection_with_stubbed_pubsub
+      open_connection
+
       assert_equal "Room#my-room:User#lifo", @connection.connection_identifier
     end
   end
 
   private
-    def open_connection_with_stubbed_pubsub
+    def open_connection
       server = TestServer.new
-      server.stubs(:pubsub).returns(stub_everything("pubsub"))
-
-      open_connection server: server
-    end
-
-    def open_connection(server:)
       env = Rack::MockRequest.env_for "/test", "HTTP_HOST" => "localhost", "HTTP_CONNECTION" => "upgrade", "HTTP_UPGRADE" => "websocket"
       @connection = Connection.new(server, env)
 
