@@ -1,5 +1,6 @@
 # frozen_string_literal: true
 
+require "active_support/testing/strict_warnings"
 require "action_cable"
 require "active_support/testing/autorun"
 require "active_support/testing/method_call_assertions"
@@ -7,11 +8,6 @@ require "active_support/testing/method_call_assertions"
 require "puma"
 require "rack/mock"
 
-begin
-  require "byebug"
-rescue LoadError
-end
-
 # Require all the stubs and models
 Dir[File.expand_path("stubs/*.rb", __dir__)].each { |file| require file }
 
