@@ -1,12 +1,14 @@
-require 'abstract_unit'
+# frozen_string_literal: true
+
+require "abstract_unit"
 
 class PeopleController < ActionController::Base
   def create
-    render text: params[:person].permitted? ? "permitted" : "forbidden"
+    render plain: params[:person].permitted? ? "permitted" : "forbidden"
   end
 
   def create_with_permit
-    render text: params[:person].permit(:name).permitted? ? "permitted" : "forbidden"
+    render plain: params[:person].permit(:name).permitted? ? "permitted" : "forbidden"
   end
 end
 
@@ -14,12 +16,12 @@ class ActionControllerPermittedParamsTest < ActionController::TestCase
   tests PeopleController
 
   test "parameters are forbidden" do
-    post :create, { person: { name: "Mjallo!" } }
+    post :create, params: { person: { name: "Mjallo!" } }
     assert_equal "forbidden", response.body
   end
 
   test "parameters can be permitted and are then not forbidden" do
-    post :create_with_permit, { person: { name: "Mjallo!" } }
+    post :create_with_permit, params: { person: { name: "Mjallo!" } }
     assert_equal "permitted", response.body
   end
 end
