@@ -9,7 +9,6 @@ class TestControllerWithExtraEtags < ActionController::Base
     "test/hello_world.erb" => "Hello world!"
   )]
 
-  def self.controller_name; "test"; end
   def self.controller_path; "test"; end
 
   etag { nil  }
@@ -93,7 +92,7 @@ class TestController < ActionController::Base
   end
 
   def conditional_hello
-    if stale?(last_modified: Time.now.utc.beginning_of_day, etag: [:foo, 123])
+    if stale?(last_modified: Time.now.utc.beginning_of_day, etag: [:foo, 123], cache_control: { no_cache: true })
       render action: "hello_world"
     end
   end
@@ -106,6 +105,15 @@ class TestController < ActionController::Base
     end
   end
 
+  def conditional_hello_with_array_of_records
+    record = Struct.new(:updated_at, :cache_key).new(Time.now.utc.beginning_of_day, "foo/123")
+    old_record = Struct.new(:updated_at, :cache_key).new(Time.now.utc.beginning_of_day.yesterday, "bar/123")
+
+    if stale?([record, old_record])
+      render action: "hello_world"
+    end
+  end
+
   def dynamic_render
     render params[:id] # => String, AC::Params
   end
@@ -214,7 +222,7 @@ class TestController < ActionController::Base
   before_action :handle_last_modified_and_etags, only: :conditional_hello_with_bangs
 
   def handle_last_modified_and_etags
-    fresh_when(last_modified: Time.now.utc.beginning_of_day, etag: [ :foo, 123 ])
+    fresh_when(last_modified: Time.now.utc.beginning_of_day, etag: [ :foo, 123 ], public: false, cache_control: { no_cache: true, public: true })
   end
 
   def head_created
@@ -288,6 +296,24 @@ class TestController < ActionController::Base
     end
   end
 
+  def cache_control_default_header_with_extras_partially_overridden_by_expires_in
+    response.headers["Cache-Control"] = "max-age=120, public, s-maxage=60, proxy-revalidate"
+    expires_in 300.seconds, public: true
+    render action: "hello_world"
+  end
+
+  def cache_control_no_store_overridden_by_expires_in
+    response.headers["Cache-Control"] = "no-store"
+    expires_in 60.seconds, public: true
+    render action: "hello_world"
+  end
+
+  def cache_control_no_store_overridden_by_expires_now
+    response.headers["Cache-Control"] = "no-store"
+    expires_now
+    render action: "hello_world"
+  end
+
   private
     def set_variable_for_layout
       @variable_for_layout = nil
@@ -369,9 +395,7 @@ class ExpiresInRenderTest < ActionController::TestCase
   def test_dynamic_render
     assert File.exist?(File.expand_path("../../test/abstract_unit.rb", __dir__))
     assert_raises ActionView::MissingTemplate do
-      assert_deprecated do
-        get :dynamic_render, params: { id: '../\\../test/abstract_unit.rb' }
-      end
+      get :dynamic_render, params: { id: '../\\../test/abstract_unit.rb' }
     end
   end
 
@@ -461,6 +485,21 @@ class ExpiresInRenderTest < ActionController::TestCase
       assert_equal Time.now.httpdate, @response.headers["Date"]
     end
   end
+
+  def test_cache_control_default_header_with_extras_partially_overridden_by_expires_in
+    get :cache_control_default_header_with_extras_partially_overridden_by_expires_in
+    assert_equal "max-age=300, public, s-maxage=60, proxy-revalidate", @response.headers["Cache-Control"]
+  end
+
+  def test_cache_control_no_store_overridden_by_expires_in
+    get :cache_control_no_store_overridden_by_expires_in
+    assert_equal "max-age=60, public", @response.headers["Cache-Control"]
+  end
+
+  def test_cache_control_no_store_overridden_by_expires_now
+    get :cache_control_no_store_overridden_by_expires_now
+    assert_equal "no-cache", @response.headers["Cache-Control"]
+  end
 end
 
 class LastModifiedRenderTest < ActionController::TestCase
@@ -499,6 +538,11 @@ class LastModifiedRenderTest < ActionController::TestCase
     assert_equal @last_modified, @response.headers["Last-Modified"]
   end
 
+  def test_responds_with_custom_cache_control_headers
+    get :conditional_hello
+    assert_equal "no-cache", @response.headers["Cache-Control"]
+  end
+
   def test_responds_with_last_modified_with_record
     get :conditional_hello_with_record
     assert_equal @last_modified, @response.headers["Last-Modified"]
@@ -528,6 +572,34 @@ class LastModifiedRenderTest < ActionController::TestCase
     assert_equal @last_modified, @response.headers["Last-Modified"]
   end
 
+  def test_responds_with_last_modified_with_array_of_records
+    get :conditional_hello_with_array_of_records
+    assert_equal @last_modified, @response.headers["Last-Modified"]
+  end
+
+  def test_request_not_modified_with_array_of_records
+    @request.if_modified_since = @last_modified
+    get :conditional_hello_with_array_of_records
+    assert_equal 304, @response.status.to_i
+    assert_predicate @response.body, :blank?
+    assert_equal @last_modified, @response.headers["Last-Modified"]
+  end
+
+  def test_request_not_modified_but_etag_differs_with_array_of_records
+    @request.if_modified_since = @last_modified
+    @request.if_none_match = '"234"'
+    get :conditional_hello_with_array_of_records
+    assert_response :success
+  end
+
+  def test_request_modified_with_array_of_records
+    @request.if_modified_since = "Thu, 16 Jul 2008 00:00:00 GMT"
+    get :conditional_hello_with_array_of_records
+    assert_equal 200, @response.status.to_i
+    assert_predicate @response.body, :present?
+    assert_equal @last_modified, @response.headers["Last-Modified"]
+  end
+
   def test_responds_with_last_modified_with_collection_of_records
     get :conditional_hello_with_collection_of_records
     assert_equal @last_modified, @response.headers["Last-Modified"]
@@ -573,6 +645,12 @@ class LastModifiedRenderTest < ActionController::TestCase
     get :conditional_hello_with_bangs
     assert_response :success
   end
+
+  def test_last_modified_with_custom_cache_control_headers
+    get :conditional_hello_with_bangs
+    assert_equal "public, no-cache", @response.headers["Cache-Control"]
+    assert_response :success
+  end
 end
 
 class EtagRenderTest < ActionController::TestCase
@@ -776,20 +854,20 @@ class HeadRenderTest < ActionController::TestCase
   def test_head_created_with_application_json_content_type
     post :head_created_with_application_json_content_type
     assert_predicate @response.body, :blank?
-    assert_equal "application/json", @response.header["Content-Type"]
+    assert_equal "application/json", @response.headers["Content-Type"]
     assert_response :created
   end
 
   def test_head_ok_with_image_png_content_type
     post :head_ok_with_image_png_content_type
     assert_predicate @response.body, :blank?
-    assert_equal "image/png", @response.header["Content-Type"]
+    assert_equal "image/png", @response.headers["Content-Type"]
     assert_response :ok
   end
 
   def test_head_respect_string_content_type
     get :head_ok_with_string_key_content_type
-    assert_equal "application/pdf", @response.header["Content-Type"]
+    assert_equal "application/pdf", @response.headers["Content-Type"]
   end
 
   def test_head_with_location_header
@@ -804,7 +882,7 @@ class HeadRenderTest < ActionController::TestCase
       set.draw do
         resources :customers
 
-        ActiveSupport::Deprecation.silence do
+        ActionDispatch.deprecator.silence do
           get ":controller/:action"
         end
       end
@@ -889,7 +967,7 @@ class HeadRenderTest < ActionController::TestCase
 
   def test_head_default_content_type
     post :head_default_content_type
-    assert_equal "text/html", @response.header["Content-Type"]
+    assert_equal "text/html", @response.headers["Content-Type"]
   end
 end
 
@@ -907,8 +985,8 @@ class LiveHeadRenderTest < ActionController::TestCase
   def setup
     super
 
-    def @controller.new_controller_thread
-      Thread.new { yield }
+    def @controller.new_controller_thread(&block)
+      Thread.new(&block)
     end
 
     def @controller.response_body=(body)
@@ -922,6 +1000,7 @@ class LiveHeadRenderTest < ActionController::TestCase
 
     @response.stream.on_error { flunk "action should not raise any errors" }
     sleep 0.2
+    pass
   end
 end
 
@@ -970,3 +1049,107 @@ class HttpCacheForeverTest < ActionController::TestCase
     assert_response :not_modified
   end
 end
+
+class HttpCacheNoStoreTest < ActionController::TestCase
+  class HttpCacheNoStoreController < ActionController::Base
+    def standalone_no_store_call
+      no_store
+      render plain: "hello world"
+    end
+
+    before_action(only: :no_store_overridden_by_expires_in) { no_store }
+    def no_store_overridden_by_expires_in
+      expires_in 30.seconds
+      render plain: "hello world"
+    end
+
+    before_action(only: :expires_in_overridden_by_no_store) { expires_in 30.seconds }
+    def expires_in_overridden_by_no_store
+      no_store
+      render plain: "hello world"
+    end
+
+    before_action(only: :no_store_overridden_by_fresh_when) { no_store }
+    def no_store_overridden_by_fresh_when
+      fresh_when(etag: "123abc")
+      render plain: "hello world"
+    end
+
+    before_action(only: :fresh_when_overridden_by_no_store) { fresh_when etag: "abc123" }
+    def fresh_when_overridden_by_no_store
+      no_store
+      render plain: "hello world"
+    end
+
+    before_action(only: :expires_now_overridden_by_no_store) { expires_now }
+    def expires_now_overridden_by_no_store
+      no_store
+      render plain: "hello world"
+    end
+
+    before_action(only: :no_store_overridden_by_expires_now) { no_store }
+    def no_store_overridden_by_expires_now
+      expires_now
+      render plain: "hello world"
+    end
+
+    def cache_control_no_cache_overridden_by_no_store
+      response.headers["Cache-Control"] = "no-cache"
+      no_store
+      render plain: "hello world"
+    end
+
+    def cache_control_public_with_max_age_overridden_by_no_store
+      response.headers["Cache-Control"] = "public, max-age=604800"
+      no_store
+      render plain: "hello world"
+    end
+  end
+
+  tests HttpCacheNoStoreController
+
+  def test_standalone_no_store_call
+    get :standalone_no_store_call
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+
+  def test_no_store_overridden_by_expires_in
+    get :no_store_overridden_by_expires_in
+    assert_equal "max-age=30, private", @response.headers["Cache-Control"]
+  end
+
+  def test_expires_in_overridden_by_no_store
+    get :expires_in_overridden_by_no_store
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+
+  def test_no_store_overridden_by_fresh_when
+    get :no_store_overridden_by_fresh_when
+    assert_equal "max-age=0, private, must-revalidate", @response.headers["Cache-Control"]
+  end
+
+  def test_fresh_when_overridden_by_no_store
+    get :fresh_when_overridden_by_no_store
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+
+  def test_expires_now_overridden_by_no_store
+    get :expires_now_overridden_by_no_store
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+
+  def test_no_store_overridden_by_expires_now
+    get :no_store_overridden_by_expires_now
+    assert_equal "no-cache", @response.headers["Cache-Control"]
+  end
+
+  def test_cache_control_no_cache_header_can_be_overridden_by_no_store
+    get :cache_control_no_cache_overridden_by_no_store
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+
+  def test_cache_control_public_with_expiration_header_can_be_overridden_by_no_store
+    get :cache_control_public_with_max_age_overridden_by_no_store
+    assert_equal "no-store", @response.headers["Cache-Control"]
+  end
+end
