@@ -130,6 +130,40 @@ module ActionDispatch
         assert_nil session.dig("one", :two)
       end
 
+      def test_id_was_for_new_session_that_does_not_exist
+        session = Session.create(store_for_session_that_does_not_exist, req, {})
+        assert_nil session.id_was
+      end
+
+      def test_id_was_for_session_that_does_not_exist_after_writing
+        session = Session.create(store_for_session_that_does_not_exist, req, {})
+        session["one"] = "1"
+        assert_nil session.id_was
+      end
+
+      def test_id_was_for_session_that_does_not_exist_after_destroying
+        session = Session.create(store_for_session_that_does_not_exist, req, {})
+        session.destroy
+        assert_nil session.id_was
+      end
+
+      def test_id_was_for_existing_session
+        session = Session.create(store, req, {})
+        assert_equal 1, session.id_was
+      end
+
+      def test_id_was_for_existing_session_after_write
+        session = Session.create(store, req, {})
+        session["one"] = "1"
+        assert_equal 1, session.id_was
+      end
+
+      def test_id_was_for_existing_session_after_destroy
+        session = Session.create(store, req, {})
+        session.destroy
+        assert_equal 1, session.id_was
+      end
+
       private
         def store
           Class.new {
@@ -146,6 +180,14 @@ module ActionDispatch
             def delete_session(env, id, options); 123; end
           }.new
         end
+
+        def store_for_session_that_does_not_exist
+          Class.new {
+            def load_session(env); [1, {}]; end
+            def session_exists?(env); false; end
+            def delete_session(env, id, options); 123; end
+          }.new
+        end
     end
 
     class SessionIntegrationTest < ActionDispatch::IntegrationTest
