@@ -1,5 +1,6 @@
 # frozen_string_literal: true
 
+require "active_support/testing/strict_warnings"
 require "active_job"
 require "support/job_buffer"
 
@@ -12,10 +13,15 @@ if ENV["AJ_INTEGRATION_TESTS"]
   require "support/integration/helper"
 else
   ActiveJob::Base.logger = Logger.new(nil)
-  ActiveJob::Base.skip_after_callbacks_if_terminated = true
   require "adapters/#{@adapter}"
 end
 
 require "active_support/testing/autorun"
 
+def adapter_is?(*adapter_class_symbols)
+  adapter_class_symbols.map(&:to_s).include? ActiveJob::Base.queue_adapter_name
+end
+
 require_relative "../../tools/test_common"
+
+ActiveJob::Base.include(ActiveJob::EnqueueAfterTransactionCommit)
