@@ -16,6 +16,7 @@ class SecurePasswordTest < ActiveModel::TestCase
     # Simulate loading an existing user from the DB
     @existing_user = User.new
     @existing_user.password_digest = BCrypt::Password.create("password", cost: BCrypt::Engine::MIN_COST)
+    @existing_user.changes_applied
   end
 
   teardown do
@@ -61,12 +62,22 @@ class SecurePasswordTest < ActiveModel::TestCase
     assert_equal ["can't be blank"], @user.errors[:password]
   end
 
-  test "create a new user with validation and password length greater than 72" do
+  test "create a new user with validation and password length greater than 72 characters" do
     @user.password = "a" * 73
     @user.password_confirmation = "a" * 73
     assert_not @user.valid?(:create), "user should be invalid"
     assert_equal 1, @user.errors.count
-    assert_equal ["is too long (maximum is 72 characters)"], @user.errors[:password]
+    assert_equal ["is too long"], @user.errors[:password]
+  end
+
+  test "create a new user with validation and password byte size greater than 72 bytes" do
+    # Create a password with 73 bytes by using a 3-byte Unicode character (e.g., "あ") 24 times, followed by a 1-byte character "a".
+    # This will result in a password length of 25 characters, but with a byte size of 73.
+    @user.password = "あ" * 24 + "a"
+    @user.password_confirmation = "あ" * 24 + "a"
+    assert_not @user.valid?(:create), "user should be invalid"
+    assert_equal 1, @user.errors.count
+    assert_equal ["is too long"], @user.errors[:password]
   end
 
   test "create a new user with validation and a blank password confirmation" do
@@ -141,7 +152,7 @@ class SecurePasswordTest < ActiveModel::TestCase
     @existing_user.password_confirmation = "a" * 73
     assert_not @existing_user.valid?(:update), "user should be invalid"
     assert_equal 1, @existing_user.errors.count
-    assert_equal ["is too long (maximum is 72 characters)"], @existing_user.errors[:password]
+    assert_equal ["is too long"], @existing_user.errors[:password]
   end
 
   test "updating an existing user with validation and a blank password confirmation" do
@@ -166,6 +177,47 @@ class SecurePasswordTest < ActiveModel::TestCase
     assert_equal ["doesn't match Password"], @existing_user.errors[:password_confirmation]
   end
 
+  test "updating an existing user with validation and a correct password challenge" do
+    @existing_user.password = "new password"
+    @existing_user.password_challenge = "password"
+    assert @existing_user.valid?(:update), "user should be valid"
+  end
+
+  test "updating an existing user with validation and a nil password challenge" do
+    @existing_user.password = "new password"
+    @existing_user.password_challenge = nil
+    assert @existing_user.valid?(:update), "user should be valid"
+  end
+
+  test "updating an existing user with validation and a blank password challenge" do
+    @existing_user.password = "new password"
+    @existing_user.password_challenge = ""
+    assert_not @existing_user.valid?(:update), "user should be invalid"
+    assert_equal 1, @existing_user.errors.count
+    assert_equal ["is invalid"], @existing_user.errors[:password_challenge]
+  end
+
+  test "updating an existing user with validation and an incorrect password challenge" do
+    @existing_user.password = "new password"
+    @existing_user.password_challenge = "new password"
+    assert_not @existing_user.valid?(:update), "user should be invalid"
+    assert_equal 1, @existing_user.errors.count
+    assert_equal ["is invalid"], @existing_user.errors[:password_challenge]
+  end
+
+  test "updating a user without dirty tracking and a correct password challenge" do
+    validatable_visitor = Class.new(Visitor) do
+      attr_accessor :untracked_digest
+      has_secure_password :untracked
+    end.new
+
+    validatable_visitor.untracked = "password"
+    assert validatable_visitor.valid?(:update), "user should be valid"
+
+    validatable_visitor.untracked_challenge = "password"
+    assert_not validatable_visitor.valid?(:update), "user should be invalid"
+  end
+
   test "updating an existing user with validation and a blank password digest" do
     @existing_user.password_digest = ""
     assert_not @existing_user.valid?(:update), "user should be invalid"
@@ -218,6 +270,26 @@ class SecurePasswordTest < ActiveModel::TestCase
     assert_equal @user, @user.authenticate_recovery_password("42password")
   end
 
+  test "authenticate should return false and not raise when password digest is blank" do
+    @user.password_digest = " "
+    assert_equal false, @user.authenticate(" ")
+  end
+
+  test "password_salt" do
+    @user.password = "secret"
+    assert_equal @user.password_digest.salt, @user.password_salt
+  end
+
+  test "password_salt should return nil when password is nil" do
+    @user.password = nil
+    assert_nil @user.password_salt
+  end
+
+  test "password_salt should return nil when password digest is nil" do
+    @user.password_digest = nil
+    assert_nil @user.password_salt
+  end
+
   test "Password digest cost defaults to bcrypt default cost when min_cost is false" do
     ActiveModel::SecurePassword.min_cost = false
 
