@@ -124,6 +124,31 @@ class LengthValidationTest < ActiveModel::TestCase
     assert_predicate t, :valid?
   end
 
+  def test_validates_length_of_using_within_with_infinite_ranges
+    [
+      [-Float::INFINITY..10, 11],
+      [-Float::INFINITY...11, 11],
+      [-Float::INFINITY.., nil],
+      [10..Float::INFINITY, 9],
+      [10...Float::INFINITY, 9],
+      [10.., 9],
+      [..10, 11],
+      [...11, 11],
+      [-Float::INFINITY..Float::INFINITY, nil],
+    ].each do |range, invalid_length|
+      Topic.clear_validators!
+      Topic.validates_length_of(:title, within: range)
+
+      t = Topic.new("title" => "a" * 10)
+      assert_predicate t, :valid?
+
+      if invalid_length
+        t.title = "a" * invalid_length
+        assert_predicate t, :invalid?
+      end
+    end
+  end
+
   def test_optionally_validates_length_of_using_within
     Topic.validates_length_of :title, :content, within: 3..5, allow_nil: true
 
@@ -299,13 +324,13 @@ class LengthValidationTest < ActiveModel::TestCase
     Topic.validates_length_of :title, within: 3..5, allow_nil: true
 
     t = Topic.new(title: "一二三四五")
-    assert t.valid?, t.errors.inspect
+    assert_predicate t, :valid?
 
     t = Topic.new(title: "一二三")
-    assert t.valid?, t.errors.inspect
+    assert_predicate t, :valid?
 
     t.title = nil
-    assert t.valid?, t.errors.inspect
+    assert_predicate t, :valid?
   end
 
   def test_validates_length_of_using_is_utf8
@@ -456,4 +481,19 @@ class LengthValidationTest < ActiveModel::TestCase
     t.title = ""
     assert_predicate t, :valid?
   end
+
+  def test_validates_length_of_using_lambda_as_maximum
+    Topic.validates_length_of :title, maximum: -> { 5 }
+
+    t = Topic.new("title" => "valid", "content" => "whatever")
+    assert_predicate t, :valid?
+
+    t.title = "notvalid"
+    assert_predicate t, :invalid?
+    assert_predicate t.errors[:title], :any?
+    assert_equal ["is too long (maximum is 5 characters)"], t.errors[:title]
+
+    t.title = ""
+    assert_predicate t, :valid?
+  end
 end
