@@ -26,10 +26,11 @@ module ActiveRecord
 
             value = value.sub(/^\((.+)\)$/, '-\1') # (4)
             case value
-            when /^-?\D+[\d,]+\.\d{2}$/  # (1)
-              value.gsub!(/[^-\d.]/, "")
-            when /^-?\D+[\d.]+,\d{2}$/  # (2)
-              value.gsub!(/[^-\d,]/, "").sub!(/,/, ".")
+            when /^-?\D*+[\d,]+\.\d{2}$/  # (1)
+              value.delete!("^-0-9.")
+            when /^-?\D*+[\d.]+,\d{2}$/  # (2)
+              value.delete!("^-0-9,")
+              value.tr!(",", ".")
             end
 
             super(value)
