@@ -9,24 +9,23 @@ module ActiveRecord
       end
 
       def type_cast_for_database(attr_name, value)
-        return value if value.is_a?(Arel::Nodes::BindParam)
         type = type_for_attribute(attr_name)
         type.serialize(value)
       end
 
       def type_for_attribute(attr_name)
-        schema_cache = connection.schema_cache
+        schema_cache = @klass.schema_cache
 
         if schema_cache.data_source_exists?(table_name)
           column = schema_cache.columns_hash(table_name)[attr_name.to_s]
-          type = connection.lookup_cast_type_from_column(column) if column
+          if column
+            type = @klass.with_connection { |connection| connection.lookup_cast_type_from_column(column) }
+          end
         end
 
         type || Type.default_value
       end
 
-      delegate :connection, to: :@klass, private: true
-
       private
         attr_reader :table_name
     end
