@@ -6,21 +6,21 @@ class PostgresqlCaseInsensitiveTest < ActiveRecord::PostgreSQLTestCase
   class Default < ActiveRecord::Base; end
 
   def test_case_insensitiveness
-    connection = ActiveRecord::Base.connection
+    connection = ActiveRecord::Base.lease_connection
 
-    attr = Default.arel_attribute(:char1)
+    attr = Default.arel_table[:char1]
     comparison = connection.case_insensitive_comparison(attr, nil)
     assert_match(/lower/i, comparison.to_sql)
 
-    attr = Default.arel_attribute(:char2)
+    attr = Default.arel_table[:char2]
     comparison = connection.case_insensitive_comparison(attr, nil)
     assert_match(/lower/i, comparison.to_sql)
 
-    attr = Default.arel_attribute(:char3)
+    attr = Default.arel_table[:char3]
     comparison = connection.case_insensitive_comparison(attr, nil)
     assert_match(/lower/i, comparison.to_sql)
 
-    attr = Default.arel_attribute(:multiline_default)
+    attr = Default.arel_table[:multiline_default]
     comparison = connection.case_insensitive_comparison(attr, nil)
     assert_match(/lower/i, comparison.to_sql)
   end
