@@ -8,7 +8,7 @@ class PostgresqlNetworkTest < ActiveRecord::PostgreSQLTestCase
   class PostgresqlNetworkAddress < ActiveRecord::Base; end
 
   setup do
-    @connection = ActiveRecord::Base.connection
+    @connection = ActiveRecord::Base.lease_connection
     @connection.create_table("postgresql_network_addresses", force: true) do |t|
       t.inet "inet_address", default: "192.168.1.1"
       t.cidr "cidr_address", default: "192.168.1.0/24"
@@ -94,6 +94,18 @@ class PostgresqlNetworkTest < ActiveRecord::PostgreSQLTestCase
     assert_match %r{t\.macaddr\s+"mac_address",\s+default: "ff:ff:ff:ff:ff:ff"}, output
   end
 
+  def test_cidr_change_prefix
+    model = PostgresqlNetworkAddress.create(cidr_address: "192.168.1.0/24")
+    model.cidr_address = "192.168.1.0/24"
+    assert_not_predicate model, :changed?
+
+    model.cidr_address = "192.168.2.0/24"
+    assert_predicate model, :changed?
+
+    model.cidr_address = "192.168.1.0/25"
+    assert_predicate model, :changed?
+  end
+
   def test_mac_address_change_case_does_not_mark_dirty
     model = PostgresqlNetworkAddress.create(mac_address: "Ab:Cd:Ef:01:02:03")
     model.mac_address = model.mac_address.swapcase
