@@ -6,7 +6,12 @@ module ActiveRecord
   module Coders
     class YAMLColumnTest < ActiveRecord::TestCase
       setup do
-        ActiveRecord::Base.use_yaml_unsafe_load = true
+        @use_yaml_unsafe_load = ActiveRecord.use_yaml_unsafe_load
+        ActiveRecord.use_yaml_unsafe_load = true
+      end
+
+      teardown do
+        ActiveRecord.use_yaml_unsafe_load = @use_yaml_unsafe_load
       end
 
       def test_initialize_takes_class
@@ -45,9 +50,11 @@ module ActiveRecord
         assert_equal "foo", coder.load("foo")
       end
 
-      def test_load_handles_other_classes
+      def test_load_raises_on_other_classes
         coder = YAMLColumn.new("attr_name")
-        assert_equal [], coder.load([])
+        assert_raises TypeError do
+          coder.load([])
+        end
       end
 
       def test_load_doesnt_swallow_yaml_exceptions
@@ -69,12 +76,18 @@ module ActiveRecord
 
     class YAMLColumnTestWithSafeLoad < YAMLColumnTest
       setup do
-        @yaml_column_permitted_classes_default = ActiveRecord::Base.yaml_column_permitted_classes
-        ActiveRecord::Base.use_yaml_unsafe_load = false
+        @use_yaml_unsafe_load = ActiveRecord.use_yaml_unsafe_load
+        @yaml_column_permitted_classes_default = ActiveRecord.yaml_column_permitted_classes
+        ActiveRecord.use_yaml_unsafe_load = false
+      end
+
+      teardown do
+        ActiveRecord.use_yaml_unsafe_load = @use_yaml_unsafe_load
+        ActiveRecord.yaml_column_permitted_classes = @yaml_column_permitted_classes_default
       end
 
       def test_yaml_column_permitted_classes_are_consumed_by_safe_load
-        ActiveRecord::Base.yaml_column_permitted_classes = [Symbol, Time]
+        ActiveRecord.yaml_column_permitted_classes = [Symbol, Time]
 
         coder = YAMLColumn.new("attr_name")
         time_yaml = YAML.dump(Time.new)
@@ -84,8 +97,56 @@ module ActiveRecord
           coder.load(time_yaml)
           coder.load(symbol_yaml)
         end
+      end
 
-        ActiveRecord::Base.yaml_column_permitted_classes = @yaml_column_permitted_classes_default
+      def test_yaml_column_permitted_classes_are_consumed_by_safe_dump
+        if Gem::Version.new(Psych::VERSION) < Gem::Version.new("5.1")
+          skip "YAML.safe_dump is either missing on unavailable on #{Psych::VERSION}"
+        end
+
+        coder = YAMLColumn.new("attr_name")
+        assert_raises(Psych::DisallowedClass) do
+          coder.dump([Time.new])
+        end
+      end
+
+      def test_yaml_column_permitted_classes_option
+        ActiveRecord.yaml_column_permitted_classes = [Symbol]
+
+        coder = YAMLColumn.new("attr_name", permitted_classes: [Time])
+        time_yaml = YAML.dump(Time.new)
+        symbol_yaml = YAML.dump(:somesymbol)
+
+        assert_nothing_raised do
+          coder.load(time_yaml)
+          coder.load(symbol_yaml)
+        end
+      end
+
+      def test_yaml_column_unsafe_load_option
+        ActiveRecord.use_yaml_unsafe_load = false
+        ActiveRecord.yaml_column_permitted_classes = []
+
+        coder = YAMLColumn.new("attr_name", unsafe_load: true)
+        time_yaml = YAML.dump(Time.new)
+        symbol_yaml = YAML.dump(:somesymbol)
+
+        assert_nothing_raised do
+          coder.load(time_yaml)
+          coder.load(symbol_yaml)
+        end
+      end
+
+      def test_yaml_column_override_unsafe_load_option
+        ActiveRecord.use_yaml_unsafe_load = true
+        ActiveRecord.yaml_column_permitted_classes = []
+
+        coder = YAMLColumn.new("attr_name", unsafe_load: false)
+        time_yaml = YAML.dump(Time.new)
+
+        assert_raises(Psych::DisallowedClass) do
+          coder.load(time_yaml)
+        end
       end
 
       def test_load_doesnt_handle_undefined_class_or_module
