@@ -6,24 +6,25 @@ require "models/liquid"
 require "models/molecule"
 require "models/numeric_data"
 require "models/electron"
+require "models/clothing_item"
 
 module ActiveRecord
   class StatementCacheTest < ActiveRecord::TestCase
     def setup
-      @connection = ActiveRecord::Base.connection
+      @connection = ActiveRecord::Base.lease_connection
     end
 
     def test_statement_cache
       Book.create(name: "my book")
       Book.create(name: "my other book")
 
-      cache = StatementCache.create(Book.connection) do |params|
+      cache = StatementCache.create(ClothingItem.lease_connection) do |params|
         Book.where(name: params.bind)
       end
 
-      b = cache.execute([ "my book" ], Book.connection)
+      b = cache.execute([ "my book" ], ClothingItem.lease_connection)
       assert_equal "my book", b[0].name
-      b = cache.execute([ "my other book" ], Book.connection)
+      b = cache.execute([ "my other book" ], ClothingItem.lease_connection)
       assert_equal "my other book", b[0].name
     end
 
@@ -31,13 +32,13 @@ module ActiveRecord
       b1 = Book.create(name: "my book")
       b2 = Book.create(name: "my other book")
 
-      cache = StatementCache.create(Book.connection) do |params|
+      cache = StatementCache.create(ClothingItem.lease_connection) do |params|
         Book.where(id: params.bind)
       end
 
-      b = cache.execute([ b1.id ], Book.connection)
+      b = cache.execute([ b1.id ], ClothingItem.lease_connection)
       assert_equal b1.name, b[0].name
-      b = cache.execute([ b2.id ], Book.connection)
+      b = cache.execute([ b2.id ], ClothingItem.lease_connection)
       assert_equal b2.name, b[0].name
     end
 
@@ -52,18 +53,18 @@ module ActiveRecord
     end
 
     def test_statement_cache_with_simple_statement
-      cache = ActiveRecord::StatementCache.create(Book.connection) do |params|
+      cache = ActiveRecord::StatementCache.create(ClothingItem.lease_connection) do |params|
         Book.where(name: "my book").where("author_id > 3")
       end
 
       Book.create(name: "my book", author_id: 4)
 
-      books = cache.execute([], Book.connection)
+      books = cache.execute([], ClothingItem.lease_connection)
       assert_equal "my book", books[0].name
     end
 
     def test_statement_cache_with_complex_statement
-      cache = ActiveRecord::StatementCache.create(Book.connection) do |params|
+      cache = ActiveRecord::StatementCache.create(ClothingItem.lease_connection) do |params|
         Liquid.joins(molecules: :electrons).where("molecules.name" => "dioxane", "electrons.name" => "lepton")
       end
 
@@ -71,7 +72,7 @@ module ActiveRecord
       molecule = salty.molecules.create(name: "dioxane")
       molecule.electrons.create(name: "lepton")
 
-      liquids = cache.execute([], Book.connection)
+      liquids = cache.execute([], ClothingItem.lease_connection)
       assert_equal "salty", liquids[0].name
     end
 
@@ -81,7 +82,7 @@ module ActiveRecord
     end
 
     def test_statement_cache_values_differ
-      cache = ActiveRecord::StatementCache.create(Book.connection) do |params|
+      cache = ActiveRecord::StatementCache.create(ClothingItem.lease_connection) do |params|
         Book.where(name: "my book")
       end
 
@@ -89,14 +90,14 @@ module ActiveRecord
         Book.create(name: "my book")
       end
 
-      first_books = cache.execute([], Book.connection)
+      first_books = cache.execute([], ClothingItem.lease_connection)
 
       3.times do
         Book.create(name: "my book")
       end
 
-      additional_books = cache.execute([], Book.connection)
-      assert first_books != additional_books
+      additional_books = cache.execute([], ClothingItem.lease_connection)
+      assert_not_equal first_books, additional_books
     end
 
     def test_unprepared_statements_dont_share_a_cache_with_prepared_statements
@@ -104,7 +105,7 @@ module ActiveRecord
       Book.create(name: "my other book")
 
       book = Book.find_by(name: "my book")
-      other_book = Book.connection.unprepared_statement do
+      other_book = Book.lease_connection.unprepared_statement do
         Book.find_by(name: "my other book")
       end
 
@@ -112,29 +113,29 @@ module ActiveRecord
     end
 
     def test_find_by_does_not_use_statement_cache_if_table_name_is_changed
-      book = Book.create(name: "my book")
+      liquid = Liquid.create(name: "salty")
 
-      Book.find_by(name: book.name) # warming the statement cache.
+      Liquid.find_by(name: liquid.name) # warming the statement cache.
 
       # changing the table name should change the query that is not cached.
-      Book.table_name = :birds
-      assert_nil Book.find_by(name: book.name)
+      Liquid.table_name = :birds
+      assert_nil Liquid.find_by(name: liquid.name)
     ensure
-      Book.table_name = :books
+      Liquid.table_name = :liquid
     end
 
     def test_find_does_not_use_statement_cache_if_table_name_is_changed
-      book = Book.create(name: "my book")
+      liquid = Liquid.create(name: "salty")
 
-      Book.find(book.id) # warming the statement cache.
+      Liquid.find(liquid.id) # warming the statement cache.
 
       # changing the table name should change the query that is not cached.
-      Book.table_name = :birds
+      Liquid.table_name = :birds
       assert_raise ActiveRecord::RecordNotFound do
-        Book.find(book.id)
+        Liquid.find(liquid.id)
       end
     ensure
-      Book.table_name = :books
+      Liquid.table_name = :liquid
     end
 
     def test_find_association_does_not_use_statement_cache_if_table_name_is_changed
