@@ -20,8 +20,10 @@ module ActiveRecord
     test "message contains no sql" do
       sql = Book.where(author_id: 96, cover: "hard").to_sql
       error = assert_raises(ActiveRecord::StatementInvalid) do
-        Book.connection.send(:log, sql, Book.name) do
-          raise MockDatabaseError
+        Book.lease_connection.send(:log, sql, Book.name) do
+          Book.lease_connection.send(:with_raw_connection) do
+            raise MockDatabaseError
+          end
         end
       end
       assert_not error.message.include?("SELECT")
@@ -31,8 +33,10 @@ module ActiveRecord
       sql = Book.where(author_id: 96, cover: "hard").to_sql
       binds = [Minitest::Mock.new, Minitest::Mock.new]
       error = assert_raises(ActiveRecord::StatementInvalid) do
-        Book.connection.send(:log, sql, Book.name, binds) do
-          raise MockDatabaseError
+        Book.lease_connection.send(:log, sql, Book.name, binds) do
+          Book.lease_connection.send(:with_raw_connection) do
+            raise MockDatabaseError
+          end
         end
       end
       assert_equal error.sql, sql
