@@ -1,4 +1,6 @@
-class ValidWithTimestampsPeopleHaveLastNames < ActiveRecord::Migration
+# frozen_string_literal: true
+
+class ValidWithTimestampsPeopleHaveLastNames < ActiveRecord::Migration::Current
   def self.up
     add_column "people", "last_name", :string
   end
