@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require_dependency "models/arunit2_model"
+require "models/arunit2_model"
 require "active_support/core_ext/object/with_options"
 
 class College < ARUnit2Model
