@@ -5,19 +5,19 @@ module ContactFakeColumns
     base.class_eval do
       establish_connection(adapter: "fake")
 
-      connection.data_sources = [table_name]
-      connection.primary_keys = {
+      lease_connection.data_sources = [table_name]
+      lease_connection.primary_keys = {
         table_name => "id"
       }
 
-      column :id,             :integer
-      column :name,           :string
-      column :age,            :integer
-      column :avatar,         :binary
-      column :created_at,     :datetime
-      column :awesome,        :boolean
-      column :preferences,    :string
-      column :alternative_id, :integer
+      column :id,             "integer"
+      column :name,           "string"
+      column :age,            "integer"
+      column :avatar,         "binary"
+      column :created_at,     "datetime"
+      column :awesome,        "boolean"
+      column :preferences,    "string"
+      column :alternative_id, "integer"
 
       serialize :preferences
 
@@ -27,7 +27,7 @@ module ContactFakeColumns
 
   # mock out self.columns so no pesky db is needed for these tests
   def column(name, sql_type = nil, options = {})
-    connection.merge_column(table_name, name, sql_type, options)
+    lease_connection.merge_column(table_name, name, sql_type, options)
   end
 end
 
@@ -37,7 +37,7 @@ end
 
 class ContactSti < ActiveRecord::Base
   extend ContactFakeColumns
-  column :type, :string
+  column :type, "string"
 
   def type; "ContactSti" end
 end
