@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require_dependency "models/arunit2_model"
+require "models/arunit2_model"
 
 class OtherDog < ARUnit2Model
   self.table_name = "dogs"
