@@ -1,5 +1,7 @@
 # frozen_string_literal: true
 
+require "shellwords"
+
 module ActiveStorage
   class Previewer::VideoPreviewer < Previewer
     class << self
@@ -8,7 +10,7 @@ module ActiveStorage
       end
 
       def ffmpeg_exists?
-        return @ffmpeg_exists if defined?(@ffmpeg_exists)
+        return @ffmpeg_exists unless @ffmpeg_exists.nil?
 
         @ffmpeg_exists = system(ffmpeg_path, "-version", out: File::NULL, err: File::NULL)
       end
@@ -18,17 +20,17 @@ module ActiveStorage
       end
     end
 
-    def preview
+    def preview(**options)
       download_blob_to_tempfile do |input|
         draw_relevant_frame_from input do |output|
-          yield io: output, filename: "#{blob.filename.base}.jpg", content_type: "image/jpeg"
+          yield io: output, filename: "#{blob.filename.base}.jpg", content_type: "image/jpeg", **options
         end
       end
     end
 
     private
       def draw_relevant_frame_from(file, &block)
-        draw self.class.ffmpeg_path, "-i", file.path, "-y", "-vframes", "1", "-f", "image2", "-", &block
+        draw self.class.ffmpeg_path, "-i", file.path, *Shellwords.split(ActiveStorage.video_preview_arguments), "-", &block
       end
   end
 end
