@@ -6,7 +6,7 @@ require "database/setup"
 class ActiveStorage::RepresentationTest < ActiveSupport::TestCase
   test "representing an image" do
     blob = create_file_blob
-    representation = blob.representation(resize: "100x100").processed
+    representation = blob.representation(resize_to_limit: [100, 100]).processed
 
     image = read_image(representation.image)
     assert_equal 100, image.width
@@ -15,7 +15,16 @@ class ActiveStorage::RepresentationTest < ActiveSupport::TestCase
 
   test "representing a PDF" do
     blob = create_file_blob(filename: "report.pdf", content_type: "application/pdf")
-    representation = blob.representation(resize: "640x280").processed
+    representation = blob.representation(resize_to_limit: [640, 280]).processed
+
+    image = read_image(representation.image)
+    assert_equal 612, image.width
+    assert_equal 792, image.height
+  end
+
+  test "representing a PDF-based Illustrator file" do
+    blob = create_file_blob(fixture: "report.pdf", filename: "file.ai", content_type: "application/illustrator")
+    representation = blob.representation(resize_to_limit: [640, 280]).processed
 
     image = read_image(representation.image)
     assert_equal 612, image.width
@@ -24,7 +33,7 @@ class ActiveStorage::RepresentationTest < ActiveSupport::TestCase
 
   test "representing an MP4 video" do
     blob = create_file_blob(filename: "video.mp4", content_type: "video/mp4")
-    representation = blob.representation(resize: "640x280").processed
+    representation = blob.representation(resize_to_limit: [640, 280]).processed
 
     image = read_image(representation.image)
     assert_equal 640, image.width
@@ -35,7 +44,7 @@ class ActiveStorage::RepresentationTest < ActiveSupport::TestCase
     blob = create_blob
 
     assert_raises ActiveStorage::UnrepresentableError do
-      blob.representation resize: "100x100"
+      blob.representation resize_to_limit: [100, 100]
     end
   end
 end
