@@ -2,7 +2,7 @@
 
 require "date"
 
-class Date #:nodoc:
+class Date # :nodoc:
   # No Date is blank:
   #
   #   Date.today.blank? # => false
@@ -11,4 +11,8 @@ class Date #:nodoc:
   def blank?
     false
   end
+
+  def present?
+    true
+  end
 end
