@@ -9,8 +9,14 @@ class CacheEntryTest < ActiveSupport::TestCase
     assert_not entry.expired?, "entry not expired"
     entry = ActiveSupport::Cache::Entry.new("value", expires_in: 60)
     assert_not entry.expired?, "entry not expired"
-    Time.stub(:now, Time.now + 61) do
-      assert entry.expired?, "entry is expired"
+    Time.stub(:now, Time.at(entry.expires_at + 1)) do
+      assert_predicate entry, :expired?, "entry is expired"
     end
   end
+
+  def test_initialize_with_expires_at
+    entry = ActiveSupport::Cache::Entry.new("value", expires_in: 60)
+    clone = ActiveSupport::Cache::Entry.new("value", expires_at: entry.expires_at)
+    assert_equal entry.expires_at, clone.expires_at
+  end
 end
