@@ -27,8 +27,8 @@ module Rails
         end
 
         private
-          def method_missing(*args, &block)
-            instance.send(*args, &block)
+          def method_missing(...)
+            instance.send(...)
           end
       end
     end
